@@ -156,6 +156,7 @@ static GstStateChangeReturn gst_tag_demux_change_state (GstElement * element,
 static gboolean gst_tag_demux_pad_query (GstPad * pad, GstQuery * query);
 static const GstQueryType *gst_tag_demux_get_query_types (GstPad * pad);
 static gboolean gst_tag_demux_get_upstream_size (GstTagDemux * tagdemux);
+static void gst_tag_demux_send_pending_events (GstTagDemux * tagdemux);
 static void gst_tag_demux_send_tag_event (GstTagDemux * tagdemux);
 static gboolean gst_tag_demux_send_new_segment (GstTagDemux * tagdemux);
 
@@ -314,8 +315,6 @@ gst_tag_demux_dispose (GObject * object)
 static gboolean
 gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
 {
-  GstPad *srcpad = NULL;
-
   if (tagdemux->priv->src_caps == NULL ||
       !gst_caps_is_equal (new_caps, tagdemux->priv->src_caps)) {
 
@@ -332,7 +331,7 @@ gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
   }
 
   if (tagdemux->priv->srcpad == NULL) {
-    srcpad = tagdemux->priv->srcpad =
+    tagdemux->priv->srcpad =
         gst_pad_new_from_template (gst_element_class_get_pad_template
         (GST_ELEMENT_GET_CLASS (tagdemux), "src"), "src");
     g_return_val_if_fail (tagdemux->priv->srcpad != NULL, FALSE);
@@ -670,8 +669,6 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
           return GST_FLOW_UNEXPECTED;
       }
       if (outbuf) {
-        GList *events;
-
         if (G_UNLIKELY (demux->priv->srcpad == NULL)) {
           gst_buffer_unref (outbuf);
           return GST_FLOW_ERROR;
@@ -687,17 +684,7 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
         }
 
         /* send any pending events we cached */
-        GST_OBJECT_LOCK (demux);
-        events = demux->priv->pending_events;
-        demux->priv->pending_events = NULL;
-        GST_OBJECT_UNLOCK (demux);
-
-        while (events != NULL) {
-          GST_DEBUG_OBJECT (demux->priv->srcpad, "sending cached %s event: %"
-              GST_PTR_FORMAT, GST_EVENT_TYPE_NAME (events->data), events->data);
-          gst_pad_push_event (demux->priv->srcpad, GST_EVENT (events->data));
-          events = g_list_delete_link (events, events);
-        }
+        gst_tag_demux_send_pending_events (demux);
 
         /* Send our own pending tag event */
         if (demux->priv->send_tag_event) {
@@ -756,8 +743,12 @@ gst_tag_demux_sink_event (GstPad * pad, GstEvent * event)
       ret = TRUE;
       break;
     }
+    case GST_EVENT_FLUSH_STOP:
+    case GST_EVENT_FLUSH_START:
+      ret = gst_pad_event_default (pad, event);
+      break;
     default:
-      if (demux->priv->need_newseg) {
+      if (demux->priv->need_newseg && GST_EVENT_IS_SERIALIZED (event)) {
         /* Cache all events if we have a pending segment, so they don't get
          * lost (esp. tag events) */
         GST_INFO_OBJECT (demux, "caching event: %" GST_PTR_FORMAT, event);
@@ -1331,6 +1322,13 @@ gst_tag_demux_src_getrange (GstPad * srcpad,
 {
   GstTagDemux *demux = GST_TAG_DEMUX (GST_PAD_PARENT (srcpad));
 
+  /* downstream in pull mode won't miss a newsegment event,
+   * but it likely appreciates other (tag) events */
+  if (demux->priv->need_newseg) {
+    gst_tag_demux_send_pending_events (demux);
+    demux->priv->need_newseg = FALSE;
+  }
+
   if (demux->priv->send_tag_event) {
     gst_tag_demux_send_tag_event (demux);
     demux->priv->send_tag_event = FALSE;
@@ -1416,6 +1414,25 @@ gst_tag_demux_get_query_types (GstPad * pad)
   return types;
 }
 
+static void
+gst_tag_demux_send_pending_events (GstTagDemux * demux)
+{
+  GList *events;
+
+  /* send any pending events we cached */
+  GST_OBJECT_LOCK (demux);
+  events = demux->priv->pending_events;
+  demux->priv->pending_events = NULL;
+  GST_OBJECT_UNLOCK (demux);
+
+  while (events != NULL) {
+    GST_DEBUG_OBJECT (demux->priv->srcpad, "sending cached %s event: %"
+        GST_PTR_FORMAT, GST_EVENT_TYPE_NAME (events->data), events->data);
+    gst_pad_push_event (demux->priv->srcpad, GST_EVENT (events->data));
+    events = g_list_delete_link (events, events);
+  }
+}
+
 static void
 gst_tag_demux_send_tag_event (GstTagDemux * demux)
 {
