@@ -131,17 +131,9 @@ MotifWmHints, MwmHints;
 static void gst_ximagesink_reset (GstXImageSink * ximagesink);
 static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     GstXImageBuffer * ximage);
-static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
-    GstXWindow * xwindow);
+static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink);
 static void gst_ximagesink_expose (GstXOverlay * overlay);
 
-/* ElementFactory information */
-static const GstElementDetails gst_ximagesink_details =
-GST_ELEMENT_DETAILS ("Video sink",
-    "Sink/Video",
-    "A standard X based videosink",
-    "Julien Moutte <julien@moutte.net>");
-
 static GstStaticPadTemplate gst_ximagesink_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -159,7 +151,9 @@ enum
   PROP_PIXEL_ASPECT_RATIO,
   PROP_FORCE_ASPECT_RATIO,
   PROP_HANDLE_EVENTS,
-  PROP_HANDLE_EXPOSE
+  PROP_HANDLE_EXPOSE,
+  PROP_WINDOW_WIDTH,
+  PROP_WINDOW_HEIGHT
 };
 
 static GstVideoSinkClass *parent_class = NULL;
@@ -719,8 +713,6 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
     }
   }
 
-  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
-
   src.w = ximage->width;
   src.h = ximage->height;
   dst.w = ximagesink->xwindow->width;
@@ -896,7 +888,7 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
 
   gst_ximagesink_xwindow_decorate (ximagesink, xwindow);
 
-  gst_x_overlay_got_xwindow_id (GST_X_OVERLAY (ximagesink), xwindow->win);
+  gst_x_overlay_got_window_handle (GST_X_OVERLAY (ximagesink), xwindow->win);
 
   return xwindow;
 }
@@ -927,16 +919,18 @@ gst_ximagesink_xwindow_destroy (GstXImageSink * ximagesink,
 }
 
 static void
-gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
-    GstXWindow * xwindow)
+gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink)
 {
   XWindowAttributes attr;
 
-  g_return_if_fail (xwindow != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
   /* Update the window geometry */
   g_mutex_lock (ximagesink->x_lock);
+  if (G_UNLIKELY (ximagesink->xwindow == NULL)) {
+    g_mutex_unlock (ximagesink->x_lock);
+    return;
+  }
 
   XGetWindowAttributes (ximagesink->xcontext->disp,
       ximagesink->xwindow->win, &attr);
@@ -1079,6 +1073,9 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         exposed = TRUE;
         break;
       case ConfigureNotify:
+        g_mutex_unlock (ximagesink->x_lock);
+        gst_ximagesink_xwindow_update_geometry (ximagesink);
+        g_mutex_lock (ximagesink->x_lock);
         configured = TRUE;
         break;
       default:
@@ -1299,6 +1296,8 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
     g_mutex_unlock (ximagesink->x_lock);
     g_free (xcontext->par);
     g_free (xcontext);
+    GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
+        ("Could not get supported pixmap formats"), (NULL));
     return NULL;
   }
 
@@ -1399,6 +1398,9 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
   g_free (ximagesink->par);
   ximagesink->par = NULL;
 
+  if (xcontext->last_caps)
+    gst_caps_replace (&xcontext->last_caps, NULL);
+
   g_mutex_lock (ximagesink->x_lock);
 
   XCloseDisplay (xcontext->disp);
@@ -1465,7 +1467,6 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   GstXImageSink *ximagesink;
   gboolean ret = TRUE;
   GstStructure *structure;
-  GstCaps *intersection;
   const GValue *par;
   gint new_width, new_height;
   const GValue *fps;
@@ -1480,15 +1481,8 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
       GST_PTR_FORMAT, ximagesink->xcontext->caps, caps);
 
   /* We intersect those caps with our template to make sure they are correct */
-  intersection = gst_caps_intersect (ximagesink->xcontext->caps, caps);
-  GST_DEBUG_OBJECT (ximagesink, "intersection returned %" GST_PTR_FORMAT,
-      intersection);
-  if (gst_caps_is_empty (intersection)) {
-    gst_caps_unref (intersection);
-    return FALSE;
-  }
-
-  gst_caps_unref (intersection);
+  if (!gst_caps_can_intersect (ximagesink->xcontext->caps, caps))
+    goto incompatible_caps;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -1559,6 +1553,11 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   return TRUE;
 
   /* ERRORS */
+incompatible_caps:
+  {
+    GST_ERROR_OBJECT (ximagesink, "caps incompatible");
+    return FALSE;
+  }
 wrong_aspect:
   {
     GST_INFO_OBJECT (ximagesink, "pixel aspect ratio does not match");
@@ -1664,6 +1663,11 @@ gst_ximagesink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
 
   ximagesink = GST_XIMAGESINK (vsink);
 
+  /* This shouldn't really happen because state changes will fail
+   * if the xcontext can't be allocated */
+  if (!ximagesink->xcontext)
+    return GST_FLOW_ERROR;
+
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
   if (GST_IS_XIMAGE_BUFFER (buf)) {
@@ -1771,9 +1775,22 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   gboolean alloc_unref = FALSE;
   gint width, height;
   GstVideoRectangle dst, src, result;
+  gboolean caps_accepted = FALSE;
 
   ximagesink = GST_XIMAGESINK (bsink);
 
+  if (G_UNLIKELY (!caps)) {
+    GST_WARNING_OBJECT (ximagesink, "have no caps, doing fallback allocation");
+    *buf = NULL;
+    ret = GST_FLOW_OK;
+    goto beach;
+  }
+
+  /* This shouldn't really happen because state changes will fail
+   * if the xcontext can't be allocated */
+  if (!ximagesink->xcontext)
+    return GST_FLOW_ERROR;
+
   GST_LOG_OBJECT (ximagesink,
       "a buffer of %d bytes was requested with caps %" GST_PTR_FORMAT
       " and offset %" G_GUINT64_FORMAT, size, caps, offset);
@@ -1805,7 +1822,6 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   }
 
   /* What is our geometry */
-  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
   dst.w = ximagesink->xwindow->width;
   dst.h = ximagesink->xwindow->height;
 
@@ -1853,10 +1869,22 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
           GST_TYPE_FRACTION, nom, den, NULL);
     }
 
+
     /* see if peer accepts our new suggestion, if there is no peer, this 
      * function returns true. */
-    if (gst_pad_peer_accept_caps (GST_VIDEO_SINK_PAD (ximagesink),
-            desired_caps)) {
+    if (!ximagesink->xcontext->last_caps ||
+        !gst_caps_is_equal (desired_caps, ximagesink->xcontext->last_caps)) {
+      caps_accepted =
+          gst_pad_peer_accept_caps (GST_VIDEO_SINK_PAD (ximagesink),
+          desired_caps);
+
+      /* Suggestion failed, prevent future attempts for the same caps
+       * to fail as well. */
+      if (!caps_accepted)
+        gst_caps_replace (&ximagesink->xcontext->last_caps, desired_caps);
+    }
+
+    if (caps_accepted) {
       /* we will not alloc a buffer of the new suggested caps. Make sure
        * we also unref this new caps after we set it on the buffer. */
       alloc_caps = desired_caps;
@@ -1870,6 +1898,7 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
           desired_caps);
       /* we alloc a buffer with the original incomming caps already in the
        * width and height variables */
+      gst_caps_unref (desired_caps);
     }
   }
 
@@ -1988,8 +2017,9 @@ gst_ximagesink_navigation_init (GstNavigationInterface * iface)
 }
 
 static void
-gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
+gst_ximagesink_set_window_handle (GstXOverlay * overlay, guintptr id)
 {
+  XID xwindow_id = id;
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
   GstXWindow *xwindow = NULL;
   XWindowAttributes attr;
@@ -2061,6 +2091,7 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
 
+  gst_ximagesink_xwindow_update_geometry (ximagesink);
   gst_ximagesink_ximage_put (ximagesink, NULL);
 }
 
@@ -2103,7 +2134,7 @@ gst_ximagesink_set_event_handling (GstXOverlay * overlay,
 static void
 gst_ximagesink_xoverlay_init (GstXOverlayClass * iface)
 {
-  iface->set_xwindow_id = gst_ximagesink_set_xwindow_id;
+  iface->set_window_handle = gst_ximagesink_set_window_handle;
   iface->expose = gst_ximagesink_expose;
   iface->handle_events = gst_ximagesink_set_event_handling;
 }
@@ -2206,6 +2237,18 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
     case PROP_HANDLE_EXPOSE:
       g_value_set_boolean (value, ximagesink->handle_expose);
       break;
+    case PROP_WINDOW_WIDTH:
+      if (ximagesink->xwindow)
+        g_value_set_uint64 (value, ximagesink->xwindow->width);
+      else
+        g_value_set_uint64 (value, 0);
+      break;
+    case PROP_WINDOW_HEIGHT:
+      if (ximagesink->xwindow)
+        g_value_set_uint64 (value, ximagesink->xwindow->height);
+      else
+        g_value_set_uint64 (value, 0);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2319,7 +2362,9 @@ gst_ximagesink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_ximagesink_details);
+  gst_element_class_set_details_simple (element_class,
+      "Video sink", "Sink/Video",
+      "A standard X based videosink", "Julien Moutte <julien@moutte.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_ximagesink_sink_template_factory));
@@ -2370,6 +2415,30 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
           "the current frame will always be drawn in response to X Expose "
           "events", TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstXImageSink:window-width
+   *
+   * Actual width of the video window.
+   *
+   * Since: 0.10.32
+   */
+  g_object_class_install_property (gobject_class, PROP_WINDOW_WIDTH,
+      g_param_spec_uint64 ("window-width", "window-width",
+          "Width of the window", 0, G_MAXUINT64, 0,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstXImageSink:window-height
+   *
+   * Actual height of the video window.
+   *
+   * Since: 0.10.32
+   */
+  g_object_class_install_property (gobject_class, PROP_WINDOW_HEIGHT,
+      g_param_spec_uint64 ("window-height", "window-height",
+          "Height of the window", 0, G_MAXUINT64, 0,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   gstelement_class->change_state = gst_ximagesink_change_state;
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_ximagesink_getcaps);
@@ -2407,24 +2476,16 @@ gst_ximagesink_get_type (void)
       (GClassInitFunc) gst_ximagesink_class_init,
       NULL,
       NULL,
-      sizeof (GstXImageSink),
-      0,
-      (GInstanceInitFunc) gst_ximagesink_init,
+      sizeof (GstXImageSink), 0, (GInstanceInitFunc) gst_ximagesink_init,
     };
     static const GInterfaceInfo iface_info = {
-      (GInterfaceInitFunc) gst_ximagesink_interface_init,
-      NULL,
-      NULL,
+      (GInterfaceInitFunc) gst_ximagesink_interface_init, NULL, NULL,
     };
     static const GInterfaceInfo navigation_info = {
-      (GInterfaceInitFunc) gst_ximagesink_navigation_init,
-      NULL,
-      NULL,
+      (GInterfaceInitFunc) gst_ximagesink_navigation_init, NULL, NULL,
     };
     static const GInterfaceInfo overlay_info = {
-      (GInterfaceInitFunc) gst_ximagesink_xoverlay_init,
-      NULL,
-      NULL,
+      (GInterfaceInitFunc) gst_ximagesink_xoverlay_init, NULL, NULL,
     };
 
     ximagesink_type = g_type_register_static (GST_TYPE_VIDEO_SINK,
