@@ -1,8 +1,8 @@
 #ifndef _GST_PLUGINS_BASE__STDINT_H
 #define _GST_PLUGINS_BASE__STDINT_H 1
 #ifndef _GENERATED_STDINT_H
-#define _GENERATED_STDINT_H "gst-plugins-base 0.10.30.4"
-/* generated using gnu compiler gcc (Debian 4.4.5-6) 4.4.5 */
+#define _GENERATED_STDINT_H "gst-plugins-base 0.10.34"
+/* generated using gnu compiler gcc (Debian 4.5.2-8) 4.5.2 */
 #define _STDINT_HAVE_STDINT_H 1
 #include <stdint.h>
 #endif
